/**
 * Property: viewer
 * {<Geoportal.Viewer>} the viewer global instance.
 */
viewer= null;

/** get current active projection definition or choose default one **/
function getProjDef() {
  var proj = getProj();
  proj = srsDefs[proj] ? srsDefs[proj] : proj = srsDefs[currentBBox.projections[0]];
  return proj;
}

/** get current projection identifier or choose default one if invalid **/
function getProj() {
   var proj = (localStorage) ? localStorage.getItem('mobile.ign.sys') : currentBBox.projections[0];
   proj = isValidSRS(proj) ? proj : currentBBox.projections[0];
   return proj;
}

function getProjName() { 
  return getProjDef().shortName;
}

/**
 * Construct a string representation of the point
 * 
 * mon_point: point to display 
 **/
function getDisplayedPosition (mon_point) {
   var point = new OpenLayers.LonLat(mon_point.lon, mon_point.lat);
   var mapProjection = new OpenLayers.Projection('EPSG:3857');
   var proj = getProjDef();
   point.transform(mapProjection, proj.proj);
   var displayedPosition;
   
   // Differentiate display depending on units 
   switch(proj.units) {
      case 'degrees':
         displayedPosition = "<b>lon: </b>" + OpenLayers.Util.getFormattedLonLat(point.lon, 'lon').replace(" ", "&nbsp;") + "<br/>  " 
                           + "<b>lat: </b>" + OpenLayers.Util.getFormattedLonLat(point.lat, 'lat').replace(" ", "&nbsp;");
         break;
      case 'm':
      default:
         displayedPosition = "<b>X: </b>" + (parseInt(100*point.lon)/100) + "<br/>  " + "<b>Y: </b>" + (parseInt(100*point.lat)/100);
   }
   return displayedPosition;
}

/**
 * Add a pointer on Mon_point coordinates
 * mon_point: OpenlLayers point to point on
 **/
function addPointer(mon_point) {

      // clean existing pointer
      if (this.vector_layer) {
        this.vector_layer.destroyFeatures();
      }

      // Create POI from given point
      var stylepoi = { externalGraphic:'img/ici.png', graphicWidth:50, graphicHeight:50 };
      var mon_poi = new OpenLayers.Feature.Vector(mon_point, null, stylepoi);

      // clean existing pointers if exist
      $(viewer.getMap().getLayersByName('Ici')).each(function() { this.destroy(); });
      // Add a vector layer for pointer and add it to map
      this.vector_layer = new OpenLayers.Layer.Vector('Ici');
      this.vector_layer.addFeatures([mon_poi]);
      viewer.getMap().addLayer(this.vector_layer);

}

function centerAndPointer (x, y) {
    var mon_point = new OpenLayers.Geometry.Point(x, y); 
    addPointer(mon_point);
    setCenter(mon_point);     
}

function setCenter(mon_point) { 
   var projection = new OpenLayers.Projection("EPSG:3857");
   var point = new OpenLayers.LonLat(mon_point.x, mon_point.y);
   viewer.getMap().setCenter(point.transform(projection, viewer.getMap().getProjectionObject()));
}

function addPopup(mon_point, locName) {
   addPopup(mon_point, locName, true);
}

function addPopup(mon_point, locName, closeOnMove) { 

   var dp = getDisplayedPosition(mon_point);
   var screen_point = viewer.map.getPixelFromLonLat(mon_point);

   $.WebAppTracing = {};
   $.WebAppTracing.mon_point = mon_point;
   $.WebAppTracing.screen_point = screen_point;
   
   
   refreshSRS();
   $("#tracingPopup").html(formatLocName(getProjName())+getDisplayedPosition(mon_point)) ; 
   $("#tracingPopup").offset({left:screen_point.x + 5, top:screen_point.y - $("#tracingPopup").height() - 20}); 

   if (!($.WebAppTracing.evreg)) {
      $.WebAppTracing.evreg = 1;
      viewer.map.events.register("move",map, function() { 
         $("#tracingPopup").offset({left: viewer.map.getPixelFromLonLat($.WebAppTracing.mon_point).x + 5, top: viewer.map.getPixelFromLonLat($.WebAppTracing.mon_point).y - $("#tracingPopup").height() - 20}); 	
      });
}
}

function refreshPopup() {

   refreshSRS();

   var popup = $("#tracingPopup");
   if (popup) {
      $("#tracingPopup").html(formatLocName(getProjName())+getDisplayedPosition($.WebAppTracing.mon_point)) ; 
  //    $("#tracingPopup").offset({left:$.WebAppTracing.screen_point.x + 5, top:$.WebAppTracing.screen_point.y - $("#tracingPopup").height() - 20}); 
   }
}

function formatLocName(locName) {
   return "<b>  "+ locName +"</b><br/>  ";
}

function determineCoucheFromLayerContext(layers){
  
  if(_.contains(layers,'ORTHOIMAGERY.ORTHOPHOTOS') && layers.length === 1){
    return 'photos';
  }

  if(layers.length === 2 &&
    _.contains(layers,'ORTHOIMAGERY.ORTHOPHOTOS') &&
    _.contains(layers,'TRANSPORTNETWORKS.ROADS')){
    return 'routes';
  }

  if(_.contains(layers,'GEOGRAPHICALGRIDSYSTEMS.PLANIGN') && layers.length === 1){
    return 'plan-ign';
  }
  
  if(_.contains(layers,'CADASTRALPARCELS.PARCELS') &&
    _.contains(layers,'ORTHOIMAGERY.ORTHOPHOTOS') &&
    layers.length === 2){
    return 'cadastre';
  }

  if(_.contains(layers,'GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40') && layers.length === 1){
    return 'etat-major';
  }

  if((_.contains(layers,'GEOGRAPHICALGRIDSYSTEMS.MAPS.3D') ||
    _.contains(layers, 'GEOGRAPHICALGRIDSYSTEMS.MAPS')) &&
    layers.length ===1){
    return 'cartes';
  }

  return undefined;
}

/**
 * Function: initMap
 * Load the application. Called when all informations have been loaded by <loadAPI>().
 */
function initMap() {


  //options for creating viewer:
  var options= {
    // default value
    territory:'WLD',
    // default value
    //projection:'EPSG:4326',EPSG:3857
    projection:'EPSG:3857',
    // default value
    //displayProjection:'IGNF:RGF93G'
    // only usefull when loading external resources
    proxy:config.xmlProxyUrl,
    // new controls if necessary
    defaultControls:null,
    // control options
    controlsOptions: {
      // Geolocation setup
      'OpenLayers.Control.Geolocate': {
        div:$("#geolocBtn").get(0),
	geolocationOptions: {
	        enableHighAccuracy: true,
                timeout:30000,
	        maximumAge: 0
	},
        //watch {Boolean} If true, position will be update regularly.
	watch:true,
//	bind:true,
        eventListeners: {
          'locationupdated':function(obj) {
            //unLoadGeoloc();
            //this.map.zoomTo(14);
            if (localStorage && localStorage.getItem('mobile.ign.geolocate') == 1 ){
            if (this.geoloc_vector_layer) {
              this.geoloc_vector_layer.destroyFeatures();
            }
            // Create a pointer on map
            var mon_point = new OpenLayers.Geometry.Point(obj.point.x, obj.point.y);
            var stylepoi = {externalGraphic:'img/ici.png',  graphicWidth:50, graphicHeight:50};
            var mon_poi = new OpenLayers.Feature.Vector(mon_point, null, stylepoi);
            // clean existing pointers if exist
            $(viewer.getMap().getLayersByName('Ici')).each(function() { this.destroy(); });
            // Add a vector layer for pointer and add it to map
            this.geoloc_vector_layer = new OpenLayers.Layer.Vector('Ici');
            this.geoloc_vector_layer.addFeatures([mon_poi]);
            viewer.getMap().addLayer(this.geoloc_vector_layer);

            var long = obj.point.x.length;

            if (obj.point.y.length>long) {
              long=obj.point.y.length;
            }
            
            setCenter(mon_point);
            if (isTracerEnabled()) {
               addPopup(viewer.getMap().getCenter(), getProjName());
            }
            }
            else {
              unLoadGeoloc();
            } 
          },
          'locationfailed': function(obj) {

            function errorfunction(error){
                    if(error.code==error.PERMISSION_DENIED) {
                        alert('Permission refusée','Veuillez autoriser le partage de votre localisation pour accéder à cette fonctionnalité.');
                        unLoadGeoloc();
                    }
                    else{
                        alert('Echec de la localisation',"Aucun GPS n'a été trouvé.\n Veuillez vérifier que la localisation est bien activée.");
                        unLoadGeoloc();
                    }
            }
            function successfunction(position){}

            navigator.geolocation.getCurrentPosition(successfunction, errorfunction); 
                unLoadGeoloc();
          },
          'locationuncapable': function(obj) {

            function errorfunction(error){
                    if(error.code==error.PERMISSION_DENIED) {
                        alert('Permission refusée','Veuillez autoriser le partage de votre localisation pour accéder à cette fonctionnalité.');
                        unLoadGeoloc();
                    }
                    else{
                        alert('Echec de la localisation',"Aucun GPS n'a été trouvé.\n Veuillez vérifier que la localisation est bien activée.");
                        unLoadGeoloc();
                    }
            }
            function successfunction(position){}

            navigator.geolocation.getCurrentPosition(successfunction, errorfunction);

                unLoadGeoloc();
          }

        },
        destroy:function() {
          OpenLayers.Event.stopObservingElement(this.div);
          OpenLayers.Control.Geolocate.prototype.destroy.apply(this,arguments);
        },
        draw:function() {
          OpenLayers.Control.Geolocate.prototype.draw.apply(this,arguments);
          OpenLayers.Event.observe(this.div,"click", OpenLayers.Function.bindAsEventListener(function() {
            $("#geolocBtn")[0].className="hidden";
            $("#geolocBtnAct")[0].className="";
            $("#lieuRech").addClass("hidden");
            $("#doLoc").removeClass("hidden");
            if (this.active) {
              this.getCurrentLocation();
            } else {
              res = this.activate();
              
              if (!res){
                //unLoadGeoloc();
              }
            }
          }, this));

        }
      }
    }
  };

  // viewer creation of type <Geoportal.Viewer>
  viewer = new Geoportal.Viewer.Mobile('map', OpenLayers.Util.extend(
    options,
    // API keys configuration variable set by <Geoportal.GeoRMHandler.getConfig>
    window.gGEOPORTALRIGHTSMANAGEMENT===undefined? {'apiKey':config.key} : gGEOPORTALRIGHTSMANAGEMENT));

  if (!viewer) {
    // problem ...
    OpenLayers.Console.error(OpenLayers.i18n('new.instance.failed'));
    return;
  }

  // remove unusefull controls
  var controlTypes = [
    "Geoportal.Control.PermanentLogo",
    "Geoportal.Control.TermsOfService",
    "OpenLayers.Control.ZoomPanel",
    "OpenLayers.Control.ZoomIn",
    "OpenLayers.Control.ZoomToMaxExtent",
    "OpenLayers.Control.ZoomOut",
    "Geoportal.Control.Logo"
  ];
  
  // Add GraphicScale control :
  var graphicscale = new Geoportal.Control.GraphicScale();
  viewer.map.addControl(graphicscale);
  
    // Enable Zoom Wheel
  //var wheelcontrol = new OpenLayers.Control.Navigation();
  if( /PhoneGap|Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) ) {
    // mobile : on ne surcharge pas le zoom.
  }else{
    // non mobile : on ajoute le zoom molette, en conservant l'effet de glissement
    var wheelcontrol = new OpenLayers.Control.Navigation({dragPanOptions:{enableKinetic:true}});
    viewer.map.addControl(wheelcontrol);
  }

 
//Add all Layers 
  viewer.addGeoportalLayers(['GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40:WMTS'], {global:{opacity:1,visibility:false,numZoomLevel:null,minZoomLevel:6,maxZoomLevel:15}});
  viewer.addGeoportalLayers(['GEOGRAPHICALGRIDSYSTEMS.MAPS:WMTS'], {global:{opacity:1,visibility:false,minZoomLevel:3,maxZoomLevel:18}});
  viewer.addGeoportalLayers(['CADASTRALPARCELS.PARCELS:WMTS'], {global:{opacity:1,visibility:false,minZoomLevel:6,maxZoomLevel:20}});
  viewer.addGeoportalLayers(['ORTHOIMAGERY.ORTHOPHOTOS:WMTS'], {global:{opacity:1,visibility:false,minZoomLevel:6,maxZoomLevel:19}});
  viewer.addGeoportalLayers(['TRANSPORTNETWORKS.ROADS:WMTS'], {global:{opacity:1,visibility:false,minZoomLevel:6,maxZoomLevel:18}});
  viewer.addGeoportalLayers(['GEOGRAPHICALGRIDSYSTEMS.PLANIGN'], {global:{opacity:1,visibility:false,minZoomLevel:6,maxZoomLevel:18}});
  updateCouche(viewer);

  $(controlTypes).each(function(i, controlType) {
    $(viewer.map.getControlsByClass(controlType)).each(function(j, c) {
      viewer.map.removeControl(c);
    });
  });
    if(config.permalink.layers){
    var couche = determineCoucheFromLayerContext(_.pluck(config.permalink.layers,'name'));
    if(!couche){
      window.displayPopup();
      couche = 'cartes';
    }
    localStorage.setItem('mobile.ign.couche', couche);
  }
    updateCouche(viewer);
  


  // get parameters from permalink or localStorage
  if (config.permalink && config.permalink.center) {
    var lonLat = OpenLayers.Projection.transform(
      new OpenLayers.Geometry.Point(
        config.permalink.center[0],config.permalink.center[1]),
      'EPSG:4326',
      viewer.getMap().getProjection());
    var zoom = viewer.getMap().getZoomForResolution(
      config.permalink.resolution * OpenLayers.METERS_PER_INCH * OpenLayers.INCHES_PER_UNIT['degrees']
    );
    viewer.getMap().setLonLatCenter(lonLat.x, lonLat.y, zoom);
  } else if (localStorage && localStorage.getItem('mobile.ign.lon') !== null && localStorage.getItem('mobile.ign.lat') !== null && localStorage.getItem('mobile.ign.zoom.value') !== null) {
    viewer.getMap().setLonLatCenter(Number(localStorage.getItem('mobile.ign.lon')),Number(localStorage.getItem('mobile.ign.lat')),Number(localStorage.getItem('mobile.ign.zoom.value')));
  } else {
    
    var width = $(window).width(), zoomLevel;
    
    // Phone
    if (width < 768) {
      zoomLevel = 5;
    }
    // Tablet and >
    else {
      zoomLevel = 6;
    }
    
    // set center at default location and zoom (Paris by default)
    viewer.getMap().setCenterAtLonLat("02° 00' 00\" E", "47° 20' 00\" N", zoomLevel);
    $('#geolocBtn').click.apply($('#geolocBtn'));
    loadParams();
  }

  gazetteerExt = new Geoportal.Control.LocationUtilityService.GeoNames(
   new Geoportal.Layer.OpenLS.Core.LocationUtilityService(
            'StreetAddress:OPENLS;Geocode',//layer name
            {
    maximumResponses:5,
    requestOptions: {
      proxy:config.xmlProxyUrl,
      method:config.geocodeMethod
    },
    formatOptions: {
      version:'1.2'
    }
  }), {
    div:OpenLayers.Util.getElement("recherche"),
    // suffix of all fields' form
    id: 'Rech',
    // place where to display results
    resultDiv: OpenLayers.Util.getElement('resultsRech'),
    fields:{
      'q0':'name',
      'c' :null,
      's' :'search',
      'w' :null
    },
    countryCode : "StreetAddress,PositionOfInterest",

    onSelectLocation: function(f) {
      closeRecherche();
      centerAndPointer(f.geometry.x, f.geometry.y);     

      if (isTracerEnabled()) {
         addPopup(viewer.getMap().getCenter(), getProjName());
      }
    },
    activate: function() {
      this.layer.selectCntrl.deactivate();
      this.layer.destroyFeatures();
      if (this.layer.map == null) {
        var ret = this.map.addLayer(this.layer);
      }
      this.loadContent(OpenLayers.Util.getElement('searchForm'));
      this.resultDiv.innerHTML= '';
      this.resultDiv.style.visibility = 'hidden';
      if (this.autoCompleteControl) {
        this.map.addControl(this.autoCompleteControl);
      }
    },
    deactivate: function () {
      this.layer.cleanQueries();
    },
    loadContent: function (form) {
      // add mapping :
      this.inputs[this.fields.q0] = OpenLayers.Util.getElement('lieuRech');
      this.buttons[this.fields.s] = OpenLayers.Util.getElement('searchGazetteer');
      // add listeners :
      var e = this.buttons[this.fields.s];
      OpenLayers.Event.observe(
        form, "keypress", OpenLayers.Function.bind(function (evt) {   
        OpenLayers.Util.getElement('resultsRech').style.display= 'block';
        $("#recherche")[0].style.height = "100%";
        $("#recherche")[0].style.maxHeight = "100%"; 
        if (evt.keyCode == 13 || evt.keyCode == 10) {
          if ($(".gpGeocodeAutoCompleteProposalHighlighted").size() == 0) {
            $("#lieuRech").blur();
            $("#map").focus();
            OpenLayers.Util.getElement('resultsRech').innerHTML= '';
            OpenLayers.Util.getElement('resultsRech').style.display= 'none';
            return this.onSearchClick.apply(this, [e, evt]);
          }
          else {
            if (evt || window.event) OpenLayers.Event.stop(evt? evt : window.event);
            var coord = $(".gpGeocodeAutoCompleteProposalHighlighted span")[0].id.split(',');
         if (coord[0]!="0" || coord[1]!="0") {
              var point = new OpenLayers.LonLat(coord[0], coord[1]);
              point.transform("EPSG:4326", viewer.getMap().getProjectionObject());
              centerAndPointer(point.lon, point.lat);
         } else {
            this.geocode();
         }
            closeRecherche();
            if (isTracerEnabled()) {
               addPopup(viewer.getMap().getCenter(), getProjName());
            }
            return false;
          }
        }
        return true;
      }, this));

      e.onclick = OpenLayers.Function.bind(this.onSearchClick, this, e);
     
      // Add AutoComplete control :
      if (this.autoCompleteOptions) {
        var autoCompleteUrl = this.getAutoCompleteUrl();
	if (!autoCompleteUrl) {
		autoCompleteUrl = viewer.map.catalogue[config.key].layers['StreetAddress:'+Geoportal.Control.AutoComplete.SERVICE_ID].url ;
	}
        var autoCompleteOptions = OpenLayers.Util.extend({
          url: autoCompleteUrl,
	  output: 'json',
          type: "StreetAddress,PositionOfInterest"
        }, this.autoCompleteOptions);
        var inputField = this.inputs[this.fields.q0];
        this.autoCompleteControl = new Geoportal.Control.AutoComplete(inputField, autoCompleteOptions);
        this.autoCompleteControl.selectProposal = function(index){
          this.highlightIndex = index+1;
          this.highlightProposal(true);
          this.hideAutoCompleteList();
          this.completeList.childNodes[index];
          var coord = $(".gpGeocodeAutoCompleteProposalHighlighted span")[0].id.split(',');
          while(this.completeList.childNodes.length>0) {
            OpenLayers.Event.stopObservingElement(this.completeList.childNodes[0]);
            this.completeList.removeChild(this.completeList.childNodes[0]);
          }
          if (coord[0] || coord[1]) {
            var point = new OpenLayers.LonLat(coord[0], coord[1]);
            point.transform("EPSG:4326", viewer.getMap().getProjectionObject());
            centerAndPointer(point.lon, point.lat);
            if (isTracerEnabled()) {
               addPopup(viewer.getMap().getCenter(), getProjName());
            }
          } else {
            gazetteerExt.geocode();
          }
          closeRecherche();
        };
      }
    },
    geocode: function() {
      var a= new Geoportal.OLS.Address(this.countryCode);
      var v= OpenLayers.String.trim(this.inputs[this.fields.q0].value);
      if (v=='') { return ; }
      if (this.autoCompleteOptions) {
        var acc= this.autoCompleteControl;
        if (acc && acc.results && acc.highlightIndex) {
          var acResult= acc.results[acc.highlightIndex-1];
        }
      }
      a.name= v;
      if (this.wImg) {
        this.wImg.style.display= '';
      }
      this.layer.GEOCODE(
        [a],
        {
          onSuccess: this.LUSSuccess,
          onFailure: this.LUSFailure,
          scopeOn: this
        });
        a.destroy();
        a= null;

    },
    LUSSuccess: function(request) {
      if (Geoportal.Control.LocationUtilityService.prototype.LUSSuccess.apply(this,arguments)===false) {
        return false;
      }
      var features= this.layer.queriedAddresses[0].features;
      features.sort(this.order);
      this.resultDiv.style.display= 'none';
      var betterAccuracy = 0.90;
      var nbIndexOk=0, nbStreetAddress=0, firstIndexOk=-1, secondIndexOk=-1;
      for (var i=0, ilen=features.length; i<ilen; i++) {
        var f = features[i];
        var state = this.keepFeature(f);
        if (state == -1) { break; } // skip remaining features
        if (state == 0)  { continue; } // skip current feature
        if (f.attributes.geocodeMatchCode) { // new result found
          nbIndexOk++;
          if (f.attributes.geocodeMatchCode.countryCode == "StreetAddress") {
            nbStreetAddress++;
          }
          if (firstIndexOk == -1) {
            firstIndexOk = i;
          } else if (secondIndexOk == -1) {
            secondIndexOk = i;
          }
        }
      }
      // We consider that the results are sorted by accuracy -> more to less
      var noDirectResult = false;
      if (nbIndexOk == 1) {
        // If we have only one result, we can go to this result.
        this.onSelectLocation(features[firstIndexOk]);
      } else if (features[firstIndexOk].attributes.geocodeMatchCode.accuracy >= betterAccuracy) {
        // If the first result is more or equal than betterAccuracy, it is possible to go to its position directly if...
        if (features[secondIndexOk].attributes.geocodeMatchCode.accuracy < betterAccuracy) {
          // If the second result is less than betterAccuracy, we go the the first result
          this.onSelectLocation(features[firstIndexOk]);
        } else if ((features[firstIndexOk].attributes.geocodeMatchCode.countryCode == "StreetAddress") && (nbStreetAddress == 1)){
          // If the first result is the only StreetAddress Result and >= betterAccuracy
          this.onSelectLocation(features[firstIndexOk]);
        } else {
          noDirectResult = true;
        }
      } else {
        noDirectResult = true;
      }
      // No direct result found ?
      if (noDirectResult) {
        if (firstIndexOk != -1) { this.onSelectLocation(features[firstIndexOk]); }
      }
      this.resultDiv.style.display= '';
    },
    closeForm: function () {
      this.layer.abortRequest();
      this.resultDiv.innerHTML = '';
      this.resultDiv.style.visible = 'hidden';
    },
    //set appropriate zoom (instead of 10 ...)
    setZoom: Geoportal.Control.LocationUtilityService.GeoNames.setZoomForBDNyme,
    autoCompleteOptions: {}
  }
  );
  viewer.getMap().addControls([gazetteerExt]);
  gazetteerExt.activate();
  gazetteerExt.autoCompleteControl.completeList = OpenLayers.Util.getElement('resultsRech');
  
  //Décalage de la liste d'auto-copmplétion en cas de simulation d'une appli mobile plein écran
  if (("standalone" in window.navigator) && window.navigator.standalone) {
    document.getElementById('header').style.top = '20px';
    document.getElementById('catHeader').style.top = '20px';
    document.getElementById('reglagesHeader').style.top = '20px';
    document.getElementById('parametersHeader').style.top = '20px';
    document.getElementById('creditsHeader').style.top = '20px';
    document.getElementById('cookiesHeader').style.top = '20px';
    document.getElementById('shareHeader').style.top = '20px';
  } else {
    document.getElementById('completeList').style.top = '38px';
  }
  
  // force map redraw on resize, avoid iOS white screen of death
  $(window).resize(function() { redrawMap(); });

  // Store current coordinate
  viewer.map.events.register("move",map, function() {
     if (localStorage) {
        localStorage.setItem('mobile.ign.lon', viewer.getMap().getCenter().lon);
        localStorage.setItem('mobile.ign.lat', viewer.getMap().getCenter().lat);
	localStorage.setItem('mobile.ign.zoom.value', viewer.getMap().getZoom());
     }
  });


  $("#loader").hide();

}

/**
 * Function : clearLayers
 */
function clearLayers() {
  var map = viewer.getMap();
  var layersName = [
    'ORTHOIMAGERY.ORTHOPHOTOS',
    'TRANSPORTNETWORKS.ROADS',
    'GEOGRAPHICALGRIDSYSTEMS.MAPS',
    'GEOGRAPHICALGRIDSYSTEMS.PLANIGN',
    'CADASTRALPARCELS.PARCELS',
    'GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40'
  ];

  $(layersName).each(function(i, layer) {
    $(map.getLayersByName(layer)).each(function() { this.setVisibility(false); });
  });

}

/**
 * Function : redrawMap
 */
function redrawMap() {
 
  var map = viewer.getMap();
  var lon = (localStorage) ? localStorage.getItem('mobile.ign.lon') : null;
  var lat = (localStorage) ? localStorage.getItem('mobile.ign.lat') : null;
  var zoom = (localStorage) ? localStorage.getItem('mobile.ign.zoom.value') : null;
  if (lon && lat && zoom) {
     var center = new OpenLayers.LonLat(lon,lat);
     map.setCenter(center,zoom,true,true);
  }
  var layersName = [
    'ORTHOIMAGERY.ORTHOPHOTOS',
    'TRANSPORTNETWORKS.ROADS',
    'GEOGRAPHICALGRIDSYSTEMS.MAPS',
    'CADASTRALPARCELS.PARCELS',
    'GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40',
    'GEOGRAPHICALGRIDSYSTEMS.PLANIGN'
  ];

  $(layersName).each(function(i, layer) {
    $(map.getLayersByName(layer)).each(function() { if (this.getVisibility()) { this.redraw(true);} });
  });

}


function updateCouche(viewer) {

  clearLayers();

  //viewer.getMap().getLayersByName("_WLD_world_")[0].maxZoomLevel = 20;

  // get layer name from localStorage
  var couche = (localStorage) ? localStorage.getItem('mobile.ign.couche') : null;
  
 // $.get('/count-maps?'+couche, function(data) {/*fake request to count map selection changes */});
  
  // get max and min authorized zoom levels  
  var maxZoom = 15;
  var minZoom = 6;

  switch (couche) {
    case "photos":
//      viewer.addGeoportalLayers(['ORTHOIMAGERY.ORTHOPHOTOS:WMTS'], {global:{opacity:1,visibility:true,minZoomLevel:3,maxZoomLevel:18}});
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setOpacity(1);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].redraw(true);
      minZoom=3;
      maxZoom=19;
      break;

    case "routes":
      //viewer.addGeoportalLayers(['ORTHOIMAGERY.ORTHOPHOTOS:WMTS'], {global:{opacity:1,visibility:true,minZoomLevel:6,maxZoomLevel:18}});
      //viewer.addGeoportalLayers(['TRANSPORTNETWORKS.ROADS:WMTS'], {global:{opacity:1,visibility:true,minZoomLevel:6,maxZoomLevel:18}});
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setOpacity(1);
      viewer.getMap().getLayersByName("TRANSPORTNETWORKS.ROADS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("TRANSPORTNETWORKS.ROADS")[0].setOpacity(1);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].redraw(true);
      viewer.getMap().getLayersByName("TRANSPORTNETWORKS.ROADS")[0].redraw(true);
      minZoom=6;
      maxZoom=19;
      break;

    case "cartes":
      //viewer.addGeoportalLayers(['GEOGRAPHICALGRIDSYSTEMS.MAPS:WMTS'], {global:{opacity:1,visibility:true,minZoomLevel:3,maxZoomLevel:18}});
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.MAPS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.MAPS")[0].setOpacity(1);
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.MAPS")[0].redraw(true);
      minZoom=3;
      maxZoom=18;
      break;


    case "plan-ign":
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.PLANIGN")[0].setVisibility(true);
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.PLANIGN")[0].setOpacity(1);
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.PLANIGN")[0].redraw(true);
      minZoom=6;
      maxZoom=18;
      break;

    case "cadastre":
//      viewer.addGeoportalLayers(['CADASTRALPARCELS.PARCELS:WMTS'], {global:{opacity:1,visibility:true,minZoomLevel:6,maxZoomLevel:20}});
//      viewer.addGeoportalLayers(['ORTHOIMAGERY.ORTHOPHOTOS:WMTS'], {global:{opacity:0.5,visibility:true,minZoomLevel:6,maxZoomLevel:18}});
      viewer.getMap().getLayersByName("CADASTRALPARCELS.PARCELS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setOpacity(0.5);
      viewer.getMap().getLayersByName("CADASTRALPARCELS.PARCELS")[0].setOpacity(1);
      viewer.getMap().getLayersByName("CADASTRALPARCELS.PARCELS")[0].redraw(true);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].redraw(true);
      minZoom=6;
      maxZoom=20;
      break;

    case "etat-major":
//      viewer.addGeoportalLayers(['GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40:WMTS'], {global:{opacity:1,visibility:true,numZoomLevel:null,minZoomLevel:6,maxZoomLevel:15}});
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40")[0].setVisibility(true);
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40")[0].setOpacity(1);
      viewer.getMap().getLayersByName("GEOGRAPHICALGRIDSYSTEMS.ETATMAJOR40")[0].redraw(true);
      minZoom=6;
      maxZoom=15;
      break;

    default:
 //     viewer.addGeoportalLayers(['ORTHOIMAGERY.ORTHOPHOTOS:WMTS'], {global:{opacity:1,visibility:true,minZoomLevel:3,maxZoomLevel:20}});
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setVisibility(true);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].setOpacity(1);
      viewer.getMap().getLayersByName("ORTHOIMAGERY.ORTHOPHOTOS")[0].redraw(true);
      minZoom=3;
      maxZoom=19;
      break;
  }

  
  // force min and max zoom for base layer
  $(viewer.getMap().getLayersBy('isBaseLayer',true)).each(function() { this.maxZoomLevel=maxZoom; this.minZoomLevel=minZoom ; });
  //viewer.getMap().getLayersByName("_WLD_world_")[0].maxZoomLevel=maxZoom;
  //viewer.getMap().getLayersByName("_WLD_world_")[0].minZoomLevel=minZoom;
  
  // restrict current zoom to not extend min and max authorized zoom among layers
  
  if (viewer.getMap().getZoom() > maxZoom) {
     viewer.getMap().zoomTo(maxZoom);
  }
  
  if (viewer.getMap().getZoom() < minZoom) {
     viewer.getMap().zoomTo(minZoom);
  }

  // Raise the POI on top
  var map = viewer.getMap(), poi_layer;
  if ((poi_layer = map.getLayersByName('Ici')[0])) {
    map.raiseLayer(poi_layer, map.layers.length);
  }
}

/**
 * Get max autorized zoom level among all layers of the current map aggregate
 **/
function getCurrentMaxZoom() {

   var maxZoom = 0;
   
   if (viewer.getMap().getLayersByClass("Geoportal.Layer.WMTS") && viewer.getMap().getLayersByClass("Geoportal.Layer.WMTS").length > 0) {
      $(viewer.getMap().getLayersByClass("Geoportal.Layer.WMTS")).each(function(i, layer) { 
          if (layer.maxZoomLevel > maxZoom) {
             maxZoom = layer.maxZoomLevel;
          }
      });
   }
   
   return maxZoom == 0 ? 22 : maxZoom;
}

/**
 * Get min autorized zoom level among all layers of the current map aggregate
 **/
function getCurrentMinZoom() {

   var minZoom = 21;
   
   if (viewer.getMap().getLayersByClass("Geoportal.Layer.WMTS") && viewer.getMap().getLayersByClass("Geoportal.Layer.WMTS").length > 0) {
      $(viewer.getMap().getLayersByClass("Geoportal.Layer.WMTS")).each(function(i, layer) { 
          if (layer.minZoomLevel < minZoom) {
             minZoom = layer.minZoomLevel;
          }
      });
   }
   
   return minZoom == 21 ? 6 : minZoom;
}

function closeRecherche() {
  $("#recherche")[0].style.height="";
  $("#recherche")[0].style.maxHeight="100px";
  $("#footer").show();
  if ((!localStorage) || localStorage.getItem('mobile.ign.zoom')==1 || localStorage.getItem('mobile.ign.zoom')==null){
    $("#zoomPlus").show();
    $("#zoomMoins").show();
  }

  OpenLayers.Util.getElement('resultsRech').innerHTML= '';
  OpenLayers.Util.getElement('resultsRech').style.display= 'none';
  
  viewer.getMap().zoomTo(14);
}

/** free geolocate controls **/
function unLoadGeoloc(){
  $("#geolocBtn").removeClass("hidden");
  $("#geolocBtnAct").addClass("hidden");
  $("#lieuRech").removeClass("hidden");
  $("#doLoc").addClass("hidden");
  viewer.getMap().getControlsByClass('OpenLayers.Control.Geolocate')[0].deactivate();
}

/**
 * Function: loadAPI
 * Load the configuration related with the API keys.
 * Called on "onload" event.
 * Call <initMap>() function to load the interface.
 */
function loadAPI() {
  // wait for all classes to be loaded
  if (checkApiLoading('loadAPI();',['OpenLayers','Geoportal','Geoportal.Viewer','Geoportal.Viewer.Mobile'])===false) {
    return;
  }

  // load API keys configuration, then load the interface
  Geoportal.GeoRMHandler.getConfig([config.key], null, config.serverUrl, {
    onContractsComplete: initMap
  });
}

