/** Show search bar **/
function toggleSearchOn() {
	$("#searchBar").show();
	$("#searchBtn").attr("src", "img/visu/recherche_on.png");
}

/** Hide search bar **/
function toggleSearchOff() {
	$("#searchBar").hide();
	$("#searchBtn").attr("src", "img/visu/recherche_off.png");
}

/** Switch search bar depending of its current state **/
function toggleSearch() {
	var searchBar = $('#searchBar');
	if (searchBar.is(':visible')) {
		toggleSearchOff();
	} else {
		toggleSearchOn();
	}
}

function toggleZoomOn() {
	$("#zoom-buttons").show();
}

function toggleZoomOff() {
	$("#zoom-buttons").hide();
}

/** Hide browser address bar on page scroll **/
function hideAddressBar() {
	setTimeout(function() {
		window.scrollTo(0, 1);
	}, 0);
}

function displayCatalog() {
	$("#catalog").show();
	$("#visu").hide();
}

function closeCatalog() {
	$("#catalog").hide();
	$("#visu").show();
}

function displayReglages() {
	refreshSRS();
	$("#reglages").show();
	$("#visu").hide();
}

function closeReglages() {
	$("#reglages").hide();
	$("#visu").show();

	if (isTracerEnabled()) {
		refreshPopup();
	}
}

/*********************************************/
/********** PERMALINKS AND SHARING ***********/
/*********************************************/

function displayShare() {
	generateShortURL();
	$("#share").show();
	$("#visu").hide();
}

function closeShare() {
	$("#share").hide();
	$("#visu").show();
}

function generatePermalink() {
	var result = getBaseUrl() + generateCenterZoomPermalinkPart() + generateLayersPermalinkPart() + "&permalink=yes";
	return result;
}

function getBaseUrl() {
	return config.permalinksBaseUrl;
	// TODO: for now, this value is hard-coded and fetched in config
}

function generateCenterZoomPermalinkPart() {
	// Get bounds of current view and transform to the right projection
	var bounds = viewer.getMap().calculateBounds();
	bounds = bounds.transform(viewer.getMap().getProjectionObject(), new OpenLayers.Projection("EPSG:4326"));

	// Calculate center coords
	var y = (bounds.top + bounds.bottom) / 2;
	var x = (bounds.right + bounds.left) / 2;

	//Then the resolution
	var resolution = viewer.getMap().getResolution() / 111319.49;
	// TODO: find a cleaner solution to get projected resolution

	return "?c=" + x + "," + y + "&z=" + resolution;
}

function generateLayersPermalinkPart() {
	var linkPart = "";
	var layersInUrl = [];

	// Get visible layers from current map: they will be in the permalink.
	for (var i = 0; i < viewer.getMap().layers.length; i++) {
		if (viewer.getMap().layers[i].hasOwnProperty('visibility') && viewer.getMap().layers[i].visibility === true) {
			var tmp = viewer.getMap().layers[i].name;
			if (tmp != undefined && Geoportal.Catalogue.LAYERNAMES[tmp]) {
				layersInUrl.push(viewer.getMap().layers[i]);
			}
		}
	}

	for (var i = 0; i < layersInUrl.length; i++) {
		var fullLayerName = Geoportal.Catalogue.LAYERNAMES[layersInUrl[i].name].key.replace("$", "::");

		//FIXME: the layer name is different in the mobile version catalogue, should be GEOGRAPHICALGRIDSYSTEMS.MAPS.3D
		// these maps also imply to add the aggregate parameter
		var layerIn3D = "";
		var aggregateParam = "";
		if (fullLayerName == "GEOGRAPHICALGRIDSYSTEMS.MAPS::GEOPORTAIL:OGC:WMTS") {
			fullLayerName = "GEOGRAPHICALGRIDSYSTEMS.MAPS.3D::GEOPORTAIL:OGC:WMTS";
			aggregateParam = "==aggregate";
		}

		linkPart = linkPart + "&l=" + fullLayerName + aggregateParam + "(" + layersInUrl[i].opacity + ")";
	}
	return linkPart;
}

function generateShortURL() {
	var link = generatePermalink();
	var hash;

	$.support.cors = true;

	$.ajax({
		type : "POST",
		crossDomain : true,
		url : config.shortUrlService,
		data : {
			"url" : link
		},
		success : function(responseData, textStatus, jqXHR) {
			hash = jqXHR.responseText;
			link = config.shortUrlFormat + hash;
			generatedPermalink = link;
		},
		error : function(responseData, textStatus, errorThrown) {
			link = generatePermalink();
			generatedPermalink = link;
		}
	});
}

function shareByMail() {

	var mailtoLink = "mailto:?subject=";
	mailtoLink += encodeURIComponent("Carte à consulter sur le Géoportail");
	
	mailtoLink += "&body=";
	mailtoLink += encodeURIComponent("Je vous invite à consulter cette carte sur le Géoportail :");
	mailtoLink += "%0D%0A";
	mailtoLink += encodeURIComponent(generatedPermalink);

	window.location.href = mailtoLink;
}

function shareByFacebook() {
	window.plugins.socialsharing.shareViaFacebook(
		'A découvrir sur le Géoportail' /* msg */, 
		 null /* img */, 
		 generatedPermalink /* url */, 
		 function() { 
			 /*console.log('share ok'); */ 
		}, 
		 function(errormsg){
			
			 cordova.InAppBrowser.open('https://m.facebook.com/sharer.php?m2w&u=' + encodeURIComponent(generatedPermalink),'_system','location=no'); 
		} 
	);
	return false;
}

function shareByTwitter() {
	var link=generatedPermalink.replace(/@/g, encodeURIComponent("@"));
	 window.plugins.socialsharing.shareViaTwitter(
		'A découvrir sur le Géoportail', 
		 null /* img */, 
		 link,
		 function() { 
		 }, 
		 function(errormsg){ 
			 cordova.InAppBrowser.open('https://twitter.com/share?url=' + encodeURIComponent(link), '_system', "location=no");
		 } 
	 );
	return false;
}

/******************************************/

function displayParameters() {
	$("#parameters").show();
	$("#reglages").hide();
}

function closeParameters() {
	$("#parameters").hide();

	refreshParams();

	$("#visu").show();

	if (isTracerEnabled()) {
		refreshPopup();
	}
}

function refreshParams() {

	// refresh search bar visibility
	if (isVisibleSearchFormAtLaunch()) {
		toggleSearchOn();
	} else {
		toggleSearchOff();
	}

	// refresh zoom buttons visibility
	if (isVisibleMapTools()) {
		$("#zoom-buttons").show();
	} else {
		$("#zoom-buttons").hide();
	}

}

function backToReglages() {

	refreshParams();

	$("#parameters").hide();
	$("#reglages").show();
}

function displayCredits() {
	$("#credits").show();
	$("#reglages").hide();
}

function closeCredits() {
	$("#credits").hide();
	$("#visu").show();
}

function backFromCredits() {
	$("#credits").hide();
	$("#reglages").show();
}

function displayCookies() {
	$("#cookies").show();
	$("#reglages").hide();
}

function closeCookies() {
	$("#cookies").hide();
	$("#visu").show();
}

function backFromCookies() {
	$("#cookies").hide();
	$("#reglages").show();
}

function displayPopup() {
	$('#popup').show();
}

function hidePopup() {
	$('#popup').hide();
}

/**
 * Parameters buttons toggle management
 * type: button identifier
 **/
function toggleOnOff(type) {
	// checkbox management
	if (type == "zoom" || type == "rech") {
		if ($("." + type + ".toggleOn").hasClass("hidden")) {
			$("." + type + ".toggleOff").addClass("hidden");
			$("." + type + ".toggleOn").removeClass("hidden");
		} else {
			$("." + type + ".toggleOff").removeClass("hidden");
			$("." + type + ".toggleOn").addClass("hidden");
		}
	}// projections selection management
	else {
		$(".param.selected").removeClass("selected");
		$("#" + type).addClass("selected");
		localStorage.setItem('mobile.ign.sys', type);
	}
}

/** Populate parameters options values from localStorage and prepare displaying parameters **/
function loadParams() {

	if (localStorage && (localStorage.getItem('mobile.ign.zoom') == 0)) {
		$(".toggleOff.zoom").removeClass("hidden");
	} else {
		$(".toggleOn.zoom").removeClass("hidden");
	}

	if (localStorage && (localStorage.getItem('mobile.ign.rech') == 0)) {
		$(".toggleOff.rech").removeClass("hidden");
	} else {
		$(".toggleOn.rech").removeClass("hidden");
	}
  if (localStorage){
     localStorage.setItem('mobile.ign.geolocate', 0);
  }

	refreshSRS();
}

/** reload available SRS to display on paramaters screen **/
function refreshSRS() {

	// clean selection
	if (localStorage && (localStorage.getItem('mobile.ign.sys') != null)) {
		$(".param.selected").removeClass("selected");
	}

	// get current SRS or assign a default one
	var selectedSRS = currentBBox.projections[0];

	if (localStorage && (localStorage.getItem('mobile.ign.sys') != null)) {
		selectedSRS = localStorage.getItem('mobile.ign.sys');
	}

	// clean container
	$("#srs div").remove();

	// search for current territory
	$.each(bboxDefs, function(i, def) {
		// bbox is valid : territory is active.
		if (isValidBBox(def.bbox)) {
			currentBBox = def;
			return false;
		}
	});

	// add selection option for each available territory srs
	$.each(currentBBox.projections, function(i, proj) {
		addSRSOption(proj);
	});

	// if selected SRS is not valid on this territory, assign the default one
	if (!isValidSRS(selectedSRS)) {
		selectedSRS = currentBBox.projections[0];
	}

	// select choosed SRS
	$("#" + selectedSRS).addClass("selected");

}

/**
 * Add SRS in option list
 * proj: string srs identifier
 **/
function addSRSOption(proj) {

	$('<div/>', {
		id : proj,
		'class' : 'param'
	}).appendTo('#srs');

	$('<div/>', {
		'class' : 'iparam border'
	}).appendTo('#' + proj);

	$('<span>' + srsDefs[proj].name + '</span>', {
		'class' : 'iparam border'
	}).appendTo('#' + proj + ' div');

	// bind listener
	$('#' + proj).click(function() {
		toggleOnOff(proj)
	});

}

/**
 * Return true if srsId maps to a valid element in active territory projections list
 * srsId: string identifier of the srs to check
 **/
function isValidSRS(srsId) {
	return ($.inArray(srsId, currentBBox.projections) > -1);
}

/**
 * Return true if the bbox is currently valid on map selection : current center part of bbox
 * bbox: OpenLayers.Bounds object to check
 **/
function isValidBBox(bbox) {
	return (bbox.containsLonLat(viewer.getMap().getCenter()));
}

function zoomOff() {
	toggleOnOff('zoom');
	if (localStorage) {
		localStorage.setItem('mobile.ign.zoom', 0);
	}
}

function zoomOn() {
	toggleOnOff('zoom');
	if (localStorage) {
		localStorage.setItem('mobile.ign.zoom', 1);
	}
}

function rechOff() {
	toggleOnOff('rech');
	if (localStorage) {
		localStorage.setItem('mobile.ign.rech', 0);
	}
}

function rechOn() {
	toggleOnOff('rech');
	if (localStorage) {
		localStorage.setItem('mobile.ign.rech', 1);
	}
}

function isVisibleSearchFormAtLaunch() {
	return ((!localStorage) || (!localStorage.getItem('mobile.ign.rech')) || (localStorage.getItem('mobile.ign.rech') == 1));
}

function isVisibleMapTools() {
	return ((!localStorage) || (!localStorage.getItem('mobile.ign.zoom')) || (localStorage.getItem('mobile.ign.zoom') == 1));
}

function zoomInMap() {
	viewer.map.zoomIn();
}

function zoomOutMap() {
	viewer.map.zoomOut();
}

function emptySearch() {
	$("#lieuRech").val("");
	$("#clearSearch").hide();
	$("#lieuRech").focus();
}

function enableTracing() {

	if (localStorage) {
		localStorage.setItem('mobile.ign.showCoordinates', 1);
	}

	$("#coordBtn").hide();
	$("#coordBtnAct").show();
        $("#tracingPopup").show();

	// Update tracer after a map move
	viewer.getMap().events.register('moveend', null, trace);

	trace();
}

function disableTracing() {

	if (localStorage) {
		localStorage.setItem('mobile.ign.showCoordinates', 0);
	}

	$("#coordBtnAct").hide();
	$("#coordBtn").show();
        $("#tracingPopup").hide();

	// Unregister updating tracer after a map move
	viewer.getMap().events.unregister('moveend', null, trace);

	// clean pointer
	var layers = viewer.getMap().getLayersBy('name', 'Ici');
	if (layers.length >= 1) {
		var layer = layers[0];
		layer.destroyFeatures();
	}

  //$('#popup').hide();
}

function enableTraking() {
  if (localStorage) {
    localStorage.setItem('mobile.ign.geolocate', 1);
  }
  viewer.getMap().getControlsByClass('OpenLayers.Control.Geolocate')[0].activate();
}

function disableTraking() {
  if (localStorage) {
    localStorage.setItem('mobile.ign.geolocate', 0);
    unLoadGeoloc();
  }
}


function trace(event) {

	var point = viewer.getMap().getCenter();
	centerAndPointer(point.lon, point.lat);

	refreshSRS();

	var proj = getProjName();
	addPopup(point, proj, false);
}

function isTracerEnabled() {
	return localStorage.getItem('mobile.ign.showCoordinates') == 1;
}

/** Display clear search control **/
function rechKeyPress() {
	$("#clearSearch").show();
}

/** Clear search after backspace if search input is empty**/
function rechKeyUp(evt) {
	if ((evt.keyCode == 8) && ($("#lieuRech").val() == "")) {
		$("#clearSearch").hide();
	}
}

function zoomClick() {
	if ($("div#toolZoom img.zoom").first().is(":visible")) {
		zoomOff();
	} else {
		zoomOn();
	}
}

function searchClick() {
	if ($("div#toolSearch img.rech").first().is(":visible")) {
		rechOff();
	} else {
		rechOn();
	}
}

/**
 * Layers catalog initialization
 **/
function initCouches() {

	// get selected layer or choose the first one as default
	if (localStorage && (localStorage.getItem('mobile.ign.couche') != null)) {
		$("#" + localStorage.getItem('mobile.ign.couche')).addClass("orange");
	} else {
		$('.layer img').first().addClass('orange');
	}

	$(".couche").click(function(event) {
		// add an orange frame around selected layer and store its value
		if ($("#l"+this.name)[0].className.indexOf("orange") == -1) {
			$(".couche").removeClass("orange");
			$("#" + this.name).addClass("orange");
			if (localStorage) {
				localStorage.setItem('mobile.ign.couche', this.name);
			}
		}

		updateCouche(viewer);
		closeCatalog();

	});
}

/** Initialization of the webApp depending on the user's parameters **/
function initWebApp() {
	
	// Ajout d'une marge noire en haut en cas de simulation d'une appli mobile plein écran
	if (("standalone" in window.navigator) && window.navigator.standalone) {
		$('html').css('padding-top','20px');
	}
	
	if (isVisibleSearchFormAtLaunch()) {
		toggleSearchOn();
	} else {
		toggleSearchOff();
	}

	if (isVisibleMapTools()) {
		toggleZoomOn();
	} else {
		toggleZoomOff();
	}

	// register all listeners for webapp controls
	$("#searchBtn").click(toggleSearch);

	$("#lieuRech").keypress(rechKeyPress);
	$("#lieuRech").keyup(function(e) {
		rechKeyUp(e);
	});

	$("#catalogBtn").click(displayCatalog);
	$("#reglagesBtn").click(displayReglages);
	$("#parametersBtn").click(displayParameters);
	$("#creditsBtn").click(displayCredits);
	$("#cookiesBtn").click(displayCookies);
	$("#shareBtn").click(displayShare);
	$("#coordBtn").click(enableTracing);
	$("#coordBtnAct").click(disableTracing);

        $("#geolocBtn").click(enableTraking);
        $("#geolocBtnAct").click(disableTraking);

	$("#catalog .closeButton").click(closeCatalog);
	$("#catalog .backButton").click(closeCatalog);

	$("#reglages .closeButton").click(closeReglages);
	$("#reglages .backButton").click(closeReglages);

	$("#parameters .closeButton").click(closeParameters);
	$("#parameters .backButton").click(backToReglages);

	$("#credits .closeButton").click(closeCredits);
	$("#credits .backButton").click(backFromCredits);

	$("#cookies .closeButton").click(closeCookies);
	$("#cookies .backButton").click(backFromCookies);

	$("#share .closeButton").click(closeShare);
	$("#share .backButton").click(closeShare);

	$("#shareByMail").click(shareByMail);
	$("#shareByFacebook").click(shareByFacebook);
	$("#shareByTwitter").click(shareByTwitter);

	$("#toolZoom").click(zoomClick);
	$("#toolSearch").click(searchClick);

	$("#clearSearch").click(emptySearch);

	$("#zoomin").click(zoomInMap);
	$("#zoomout").click(zoomOutMap);

	$('#popup-ok-button').click(hidePopup);

	hideAddressBar();
	initSRS();
	initCouches();

	coordinatesSRS = (localStorage && (localStorage.getItem('mobile.ign.coordinatesSRS') != null)) ? localStorage.getItem('mobile.ign.coordinatesSRS') : "geo";
	localStorage.setItem('mobile.ign.sys', currentBBox.projections[0]);
	localStorage.setItem('mobile.ign.showCoordinates', 0);

	config.permalink = config.context || new PermaLink().readLink(window.location.href);
}

/** user's default parameters **/
var visibleSearchFormAtLaunch = true;
var visibleMapTools = true;
var coordinatesSRS = true;

/** generated permalink (value refreshed on demand, such as click on share button) **/
var generatedPermalink = "";

/**
 * Function called on dom loaded
 **/
$(function() {
	initWebApp();
	// loadAPI once webapp is fully initialized
	loadAPI();
});
